package main

import (
	"bufio"
	"encoding/json"
	"flag"
	"fmt"
	"os"

	"github.com/inverse-inc/go-utils/sharedutils"

	"git.inverse.ca/inverse/fingerbank-collector/common"
	"git.inverse.ca/inverse/fingerbank-collector/fingerbankclient"
)

var fingerbankApiKeyParam = flag.String("fingerbank-api-key", "", "The Fingerbank API key to use to communicate with the API")
var dumpFname = flag.String("dump", "dump-collector.json", "The collector dump to analyse")
var collector = fccommon.NewFingerbankCollector(*fingerbankApiKeyParam)

var wantedDevices = map[int]func(ed fccommon.EndpointData){
	0:   func(fccommon.EndpointData) {},
	1:   func(fccommon.EndpointData) {},
	5:   func(fccommon.EndpointData) {},
	202: handle202,
}

func handle202(ed fccommon.EndpointData) {
}

func init() {
	flag.Parse()
	fingerbankclient.FingerbankKey = *fingerbankApiKeyParam
}

func main() {
	fi, err := os.Open(*dumpFname)
	if err != nil {
		panic(err)
	}
	defer func() {
		if err := fi.Close(); err != nil {
			panic(err)
		}
	}()
	r := bufio.NewReader(fi)

	var eds []fccommon.EndpointData
	json.NewDecoder(r).Decode(&eds)

	concChan := make(chan int, 10)
	for _, ed := range eds {
		concChan <- 1
		go func(ed fccommon.EndpointData) {
			analyseEndpointData(ed)
			<-concChan
		}(ed)
	}
}

func analyseEndpointData(ed fccommon.EndpointData) {
	cloudAPIResult, err := fingerbankclient.Interrogate(ed.ToInterrogateQuery(collector))
	sharedutils.CheckError(err)
	if handler, ok := wantedDevices[cloudAPIResult.Device.Id]; ok {
		if cloudAPIResult.Score < 60 && ed.HasBehavioralData() {
			fmt.Println(ed.UUID, "is part of the wanted devices:", cloudAPIResult.Device.Id)
			handler(ed)
		}
	}
}
