package api

import (
	"net/http"

	fclog "git.inverse.ca/inverse/fingerbank-collector/log"
	"github.com/gin-gonic/gin"
)

func ClusterIdMiddleware() gin.HandlerFunc {
	logger := fclog.Logger("")
	return func(c *gin.Context) {

		var srcPeerId string
		var dstPeerId string

		if peerIdQuery := c.Request.URL.Query()["source-peer-id"]; len(peerIdQuery) > 0 {
			srcPeerId = peerIdQuery[0]
		} else {
			logger.Debug("source-peer-id wasn't provided, won't do any validation of the peers IDs")
		}

		if peerIdQuery := c.Request.URL.Query()["destination-peer-id"]; len(peerIdQuery) > 0 {
			dstPeerId = peerIdQuery[0]
		} else {
			logger.Debug("destination-peer-id wasn't provided, won't do any validation of the peers IDs")
		}

		if srcPeerId == "" || dstPeerId == "" {
			c.Next()
			return
		}

		if srcPeerId != collector.ReplicationController.UUID.String() {
			c.JSON(http.StatusUnprocessableEntity, gin.H{"error": "The source-peer-id that was provided doesn't match this replication controller ID"})
			c.Abort()
			return
		}

		_, err1 := collector.ReplicationController.FindInboundPeer(dstPeerId)
		_, err2 := collector.ReplicationController.FindOutboundPeer(dstPeerId)
		if err1 != nil && err2 != nil {
			c.JSON(http.StatusUnprocessableEntity, gin.H{"error": "The destination-peer-id doesn't match any peer known by this collector"})
			c.Abort()
			return
		}

		c.Next()
	}
}
