package api

import (
	"fmt"
	"net/http"
	"strings"

	"github.com/gin-gonic/gin"
)

// The authorization token for usage of the API
var AuthorizationToken string

func TokenAuthMiddleware() gin.HandlerFunc {
	return func(c *gin.Context) {
		// Early return if there is no token (everybody is accepted)
		if AuthorizationToken == "" {
			c.Next()
			return
		}

		var token string
		tokenHeaders := c.Request.Header["Authorization"]
		if len(tokenHeaders) >= 1 {
			authHeader := tokenHeaders[0]
			tokenParts := strings.Split(authHeader, " ")
			if len(tokenParts) != 2 || tokenParts[0] != "Token" {
				msg := fmt.Sprintf("Invalid Authorization header provided: %q", authHeader)
				c.JSON(http.StatusUnauthorized, gin.H{"error": msg})
				c.Abort()
			}

			token = tokenParts[1]

			if token != AuthorizationToken {
				msg := fmt.Sprintf("Invalid authorization token %q", token)
				c.JSON(http.StatusUnauthorized, gin.H{"error": msg})
				c.Abort()
			}

			// If we're here then we're good to go
			c.Next()

		} else if c.Query("format") == "html" {
			// Accept the token in a key parameter when outputing HTML
			token = c.Query("key")
		} else {
			msg := fmt.Sprintf("No token provided")
			c.JSON(http.StatusUnauthorized, gin.H{"error": msg})
			c.Abort()
		}
	}
}
