#!/bin/bash

PATH="/usr/local/go/bin:$PATH"

MODE="$1"
MYSRC="$2"
BINDST="$3"
DEBPATH="$4"

function usage {
  echo "----------------------------------------------------------------------------"
  echo "Usage        : build-go.sh test|build [args]"
  echo "Build usage  : build-go.sh build /path/to/sources /path/to/built/binaries"
  echo "Test usage   : build-go.sh test /path/to/sources"
  echo "----------------------------------------------------------------------------"
}

function test_mode {
  [[ "$MODE" == "test" ]]
  return $?
}

function build_mode {
  [[ "$MODE" == "build" ]]
  return $?
}

function install_go {
    # set up GOINSTALL
    if [ -z "$DEBPATH" ]; then
        export GOINSTALL="`mktemp -d`"
    else
        export GOINSTALL="`mktemp -d -p $DEBPATH`"
    fi

    GOROOT="$GOINSTALL/go"

    echo "Installing Golang to $GOINSTALL."
    GOVER=$(egrep -o 'go[0-9]+\.[0-9]+\.[0-9]+' ../config.mk)
    if ! [ -f go$GOVER.linux-amd64.tar.gz ]; then
        wget https://dl.google.com/go/go$GOVER.linux-amd64.tar.gz
    fi

    tar -C $GOINSTALL -xzf go$GOVER.linux-amd64.tar.gz
    PATH="$GOINSTALL/go/bin:$PATH"
}

function export_go_vars {
    # set up GOPATH
    if [ -z "$DEBPATH" ]; then
        export GOPATH=`mktemp -d`
    else
        export GOPATH=`mktemp -d -p $DEBPATH`
    fi
    export GOROOT
    export GOPATH

    export GOBIN="$GOPATH/bin"
}

# Exit hook to cleanup the tmp GOPATH when exiting
function cleanup {
  rm -rf "$GOPATH"
  rm -rf "$GOINSTALL"
}

if ! test_mode && ! build_mode ; then
  echo "!!!!!! - You need to specify a valid mode (test|build)"
  usage
  exit 1
fi

if ! [ -d "$MYSRC" ]; then
  echo "!!!!!! - The source directory specified ($MYSRC) doesn't exist"
  usage
  exit 1
else
  echo "Building golang sources from $MYSRC"
fi

if build_mode && ! [ -d "$BINDST" ]; then
  echo "!!!!!! - The binary destination specified ($BINDST) doesn't exist."
  usage
  exit 1
else
  echo "Will place binaries in $BINDST"
fi

set -x
trap cleanup EXIT

# Setup
if [ -d /usr/local/go ]; then
    "/usr/local/go exists, skip installation"
else
    install_go
    export_go_vars
fi

cd "$GOPATH"
GOPATHSRC="$GOPATH/src/git.inverse.ca/inverse/fingerbank-collector"
mkdir -p $GOPATHSRC

cp -a $MYSRC/* $GOPATHSRC/

cd "$GOPATHSRC"

if [ -z "$MAKE_TARGET" ]; then
  MAKE_TARGET="fingerbank-collector-classic-pf"
fi

if build_mode; then
  # Create any binaries here and make sure to move them to the BINDST specified
  rm -f fingerbank-collector
  make $MAKE_TARGET
  mv fingerbank-collector $BINDST/
elif test_mode; then
  go test ./...  
fi
