package fccommon

import (
	"encoding/hex"
	"errors"
	"net"
	"time"

	"github.com/mdlayher/arp"
)

// Get the MAC address for an IP using an ARP request
func ARPFindMAC(ift *net.Interface, ip string) (string, error) {
	// Only try it for valid IPv4 addresses
	if !ValidIP(ip) {
		return "", errors.New("Not a valid IPv4 address")
	}

	client, err := arp.Dial(ift)
	if err != nil {
		return "", err
	}

	client.SetDeadline(time.Now().Add(1 * time.Second))

	macBytes, err := client.Resolve(net.ParseIP(ip))
	client.Close()

	if err != nil {
		return "", err
	}

	hexMac := make([]byte, hex.EncodedLen(len(macBytes)))
	hex.Encode(hexMac, macBytes)

	return string(hexMac), nil
}

// Get the MAC address for an IP while making sure its not the router MAC
// In some case, arp requests for unknown IPs will always be replied by the router with its own MAC
// This does a lookup of a fake example IP and makes sure that the reply it gets for the real IP address doesn't contain the same MAC
// That's maybe Semaan not understanding ARP correctly, so feel free to change this. For reference, this behavior was seen in VLAN 20 at Inverse HQ
func ARPFindMACNotRouter(ift *net.Interface, ip string) (string, error) {
	// Looking up an IP in the test range that should never be addressed by anyone (usage is for doc and examples)
	// This will return us the IP address of the router
	// We ignore errors from this
	router, _ := ARPFindMAC(ift, "198.51.100.1")

	mac, err := ARPFindMAC(ift, ip)

	if err != nil {
		return "", nil
	}

	if mac != router {
		return mac, nil
	} else {
		return "", nil
	}
}
