package fccommon

import (
	"testing"
	"time"

	"git.inverse.ca/inverse/fingerbank-collector/fingerbankclient"

	"github.com/inverse-inc/go-utils/sharedutils"
)

func TestBuildNBAPolicies(t *testing.T) {
	fc := NewFingerbankCollector("")

	fingerbankclient.FingerbankKey = sharedutils.EnvOrDefault("FINGERBANK_API_KEY", "")

	nbaPolicies, err := fc.buildNBAPolicies([]byte(`
[policy1]
# which devices (and childs) to take into consideration in the policy. empty means all
devices_included=
# which devices (and childs) to exclude from the devices_included
devices_excluded=1,5
# whether or not to check for hosts talking with blacklisted ips
watch_blacklisted_ips=enabled
# which ips (can be cidr) to ignore in the blacklisted ips
whitelisted_ips=192.168.1.0/24,0.0.0.0/1
# how many blacklisted hosts can a device talk to before being flagged
blacklisted_ip_hosts_threshold=1
# the window to look into for the blacklisted ips
blacklisted_ip_hosts_window=1h
# which attributes to match against the profile data
watched_device_attributes=dhcp_fingerprint,dhcp_vendor,hostname
# difference thresuold that is allowed in the attributes
device_attributes_diff_score=50
# overrides for device attributes
device_attributes_diff_threshold_overrides=dhcp_fingerprint:0,user_agents:69,somethinginvalid:8,invalidnumber:dinde

[policy2]
# which devices (and childs) to take into consideration in the policy. empty means all
devices_included=1,5
# which devices (and childs) to exclude from the devices_included
devices_excluded=
# whether or not to check for hosts talking with blacklisted ips
watch_blacklisted_ips=disabled
# which ips (can be cidr) to ignore in the blacklisted ips
whitelisted_ips=
# how many blacklisted hosts can a device talk to before being flagged
blacklisted_ip_hosts_threshold=0
# the window to look into for the blacklisted ips
blacklisted_ip_hosts_window=1h
# which attributes to match against the profile data
watched_device_attributes=
# difference thresuold that is allowed in the attributes
device_attributes_diff_score=50
blacklisted_ports=80,443,1000-2000
`))

	if err != nil {
		t.Error("Error while creating NBA policies " + err.Error())
	}

	if len(nbaPolicies) != 2 {
		t.Fatal("Wrong length for NBA policies")
	}

	pol := nbaPolicies[0]

	if len(pol.DevicesIncluded) != 0 {
		t.Error("Unexpected length for attribute")
	}

	if len(pol.DevicesExcluded) == 0 {
		t.Error("Unexpected length for attribute")
	}

	if pol.WatchBlacklistedIPs != "enabled" {
		t.Error("Unexpected value for attribute")
	}

	if len(pol.WhitelistedIPs) != 2 {
		t.Error("Unexpected length for attribute")
	}

	if pol.BlacklistedIpHostsThreshold != 1 {
		t.Error("Unexpected value for attribute")
	}

	if pol.BlacklistedIpHostsWindow != 1*time.Hour {
		t.Error("Unexpected value for attribute")
	}

	if len(pol.WatchedDeviceAttributes) != 3 {
		t.Error("Unexpected length for attribute")
	}

	if pol.DeviceAttributesDiffScore != 50 {
		t.Error("Unexpected value for attribute")
	}

	if len(pol.DeviceAttributesDiffThresholdOverrides) != 2 {
		t.Error("Unexpected length for attribute")
	}

	pol = nbaPolicies[1]

	if len(pol.DevicesIncluded) == 0 {
		t.Error("Unexpected length for attribute")
	}

	if len(pol.DevicesExcluded) != 0 {
		t.Error("Unexpected length for attribute")
	}

	if pol.WatchBlacklistedIPs != "disabled" {
		t.Error("Unexpected value for attribute")
	}

	if len(pol.WhitelistedIPs) != 0 {
		t.Error("Unexpected length for attribute")
	}

	if pol.BlacklistedIpHostsThreshold != 0 {
		t.Error("Unexpected value for attribute")
	}

	if pol.BlacklistedIpHostsWindow != 1*time.Hour {
		t.Error("Unexpected value for attribute")
	}

	if len(pol.WatchedDeviceAttributes) != 0 {
		t.Error("Unexpected length for attribute")
	}

	if pol.DeviceAttributesDiffScore != 50 {
		t.Error("Unexpected value for attribute")
	}

	if len(pol.DeviceAttributesDiffThresholdOverrides) != 0 {
		t.Error("Unexpected length for attribute")
	}

	if len(pol.BlacklistedPorts) != 1003 {
		t.Error("Unexpected length for attribute")
	}

	for _, port := range []uint16{80, 443, 1000, 1001, 2000} {
		if pol.BlacklistedPorts[port] != true {
			t.Error("Unexpected value")
		}
	}

}
