//go:build dpi_godpi

package fccommon

import (
	"flag"
	"fmt"

	"git.inverse.ca/inverse/fingerbank-collector/dpi"
	"github.com/inverse-inc/go-utils/sharedutils"
)

var dpiIgnoreSignaturesBase = map[string]bool{
	"":            true,
	"no_firstpkt": true,
	"no_payload":  true,
	"unknown":     true,
	"unknown_tcp": true,
	"unknown_udp": true,
}

var dpiIgnoreSignatures = map[string]bool{}

var godpiLPI = flag.Bool(
	"godpi-lpi",
	sharedutils.IsEnabled(sharedutils.EnvOrDefault("COLLECTOR_GODPI_LPI", "false")),
	`Whether or not the LPI (libprotoident) engine should be enabled in go-dpi (COLLECTOR_GODPI_LPI environment variable)`,
)

var godpiNDPI = flag.Bool(
	"godpi-ndpi",
	sharedutils.IsEnabled(sharedutils.EnvOrDefault("COLLECTOR_GODPI_NDPI", "true")),
	`Whether or not the nDPI engine should be enabled in go-dpi (COLLECTOR_GODPI_NDPI environment variable)`,
)

func init() {
	for k, v := range dpiIgnoreSignaturesBase {
		dpiIgnoreSignatures[dpi.LPIPrefix+k] = v
		dpiIgnoreSignatures[dpi.NDPIPrefix+k] = v
	}
}

func (fc *FingerbankCollector) initDPI() {
	var err error
	fc.dpiEngine, err = dpi.CreateGodpiEngine(dpi.GodpiConfig{
		WithLPI:  *godpiLPI,
		WithNDPI: *godpiNDPI,
		Logger:   fc.logger,
	})
	if err != nil {
		fc.logger.Error(fmt.Sprintf("Unable to create DPI engine: %s", err))
		fc.dpiEngine = nil
		return
	}
	fc.dpiEngine.SetCallback(fc.processPacketDPI)
}
