package fccommon

import (
	"regexp"
	"time"

	cache "github.com/patrickmn/go-cache"
)

var DefaultHostValidator HostValidator

var hvCache = cache.New(5*time.Minute, 10*time.Minute)

func init() {
	DefaultHostValidator = NewHostValidator(make([]string, 0), make([]*regexp.Regexp, 0))
}

type HostValidator struct {
	validHosts       map[string]bool
	validHostsRegexp []*regexp.Regexp
}

func NewHostValidator(validHosts []string, validHostsRegexp []*regexp.Regexp) HostValidator {
	validHostsMap := make(map[string]bool)
	for _, s := range validHosts {
		validHostsMap[s] = true
	}
	return HostValidator{validHosts: validHostsMap, validHostsRegexp: validHostsRegexp}
}

func (hv HostValidator) Validate(host string) bool {
	// If there is nothing to match, then everything matches
	if len(hv.validHosts) == 0 && len(hv.validHostsRegexp) == 0 {
		return true
	}

	if o, found := hvCache.Get(host); found {
		return o.(bool)
	}

	valid := false
	if _, ok := hv.validHosts[host]; ok {
		valid = true
	} else {
		for _, re := range hv.validHostsRegexp {
			if re.MatchString(host) {
				valid = true
				break
			}
		}
	}

	hvCache.Set(host, valid, cache.DefaultExpiration)
	return valid
}
