package fccommon

import (
	"reflect"

	"git.inverse.ca/inverse/fingerbank-collector/elru"
	"git.inverse.ca/inverse/fingerbank-collector/timedlock"
	"github.com/inverse-inc/go-utils/sharedutils"
)

type IpHostCache struct {
	lock  *timedlock.RWLock
	cache *elru.EncodableLRU
}

func NewIpHostCache(maxSize int) *IpHostCache {
	cache, err := elru.NewEncodableLRU(maxSize)
	sharedutils.CheckError(err)

	ihc := &IpHostCache{
		cache: cache,
		lock:  timedlock.NewRWLock(),
	}

	return ihc
}

func (ihc *IpHostCache) Add(ip, host string) bool {
	lockid := ihc.lock.Lock()
	defer ihc.lock.Unlock(lockid)

	_, found := ihc.Get(ip)
	ihc.cache.Add(ip, host)
	return found
}

func (ihc *IpHostCache) Get(ip string) (string, bool) {
	if o, found := ihc.cache.Get(ip); found {
		return o.(string), found
	} else {
		return "", found
	}
}

func (ihc *IpHostCache) ToMap() map[string]string {
	m := map[string]string{}
	for _, k := range ihc.cache.Keys() {
		v, found := ihc.cache.Get(k)
		// The entry got expired by the time we fetched it
		if !found {
			continue
		}
		m[k.(string)] = v.(string)
	}
	return m
}

func (ihc *IpHostCache) MarshalJSON() ([]byte, error) {
	return ihc.cache.MarshalJSON()
}

func (ihc *IpHostCache) UnmarshalJSON(data []byte) error {
	ihc.cache = elru.NewEncodableLRUForUnmarshal(reflect.TypeOf(""))
	ihc.lock = timedlock.NewRWLock()
	return ihc.cache.UnmarshalJSON(data)
}
