package elru

import (
	"encoding/json"
	"reflect"

	lru "github.com/hashicorp/golang-lru"
)

type EncodableLRU struct {
	*lru.ARCCache
	maxSize   int
	valueType reflect.Type
}

func NewEncodableLRU(maxSize int) (*EncodableLRU, error) {
	elru := &EncodableLRU{
		maxSize: maxSize,
	}
	var err error
	elru.ARCCache, err = lru.NewARC(elru.maxSize)
	return elru, err
}

func NewEncodableLRUForUnmarshal(valueType reflect.Type) *EncodableLRU {
	return &EncodableLRU{valueType: valueType}
}

func (elru *EncodableLRU) MarshalJSON() ([]byte, error) {
	data := map[string]interface{}{}
	for _, k := range elru.Keys() {
		v, found := elru.Get(k)
		if !found {
			// was expired by the time we fetched it
			continue
		}
		data[k.(string)] = v
	}
	return json.Marshal(map[string]interface{}{
		"data":     data,
		"max_size": elru.maxSize,
	})
}

func (elru *EncodableLRU) UnmarshalJSON(data []byte) error {
	m := struct {
		Data    map[string]json.RawMessage `json:"data"`
		MaxSize int                        `json:"max_size"`
	}{}
	err := json.Unmarshal(data, &m)
	if err != nil {
		return err
	}

	elru.maxSize = m.MaxSize
	elru.ARCCache, err = lru.NewARC(elru.maxSize)
	if err != nil {
		return err
	}
	for k, v := range m.Data {
		var t reflect.Type
		isPtr := false
		if elru.valueType.Kind() == reflect.Ptr || elru.valueType.Kind() == reflect.Interface {
			t = elru.valueType.Elem()
			isPtr = true
		} else {
			t = elru.valueType
		}

		val := reflect.New(t)
		o := val.Interface()
		json.Unmarshal(v, o)
		if isPtr {
			elru.Add(k, o)
		} else {
			elru.Add(k, reflect.ValueOf(o).Elem().Interface())
		}
	}

	return nil
}

func (elru *EncodableLRU) MaxSize() int {
	return elru.maxSize
}
