package fork

import (
	"fmt"
	"os"
	"syscall"
)

func NullOutFd() {
}

// Allows to close all the file descriptors that aren't STDIN, STDOUT or STDERR
// If the 3 above should be closed, then closeStd should be passed as true in ForkToDo
func CloseAllNonStdFd() {
	var limit syscall.Rlimit
	if err := syscall.Getrlimit(syscall.RLIMIT_NOFILE, &limit); err != nil {
		panic(err)
	}

	f, err := os.Open("/dev/null")
	if err != nil {
		panic(err)
	}
	devNullFd := f.Fd()

	for i := uint64(3); i < limit.Cur; i++ {
		syscall.Dup2(int(devNullFd), int(i))
		syscall.Close(int(i))
	}
}

func ForkTodo(chdir bool, closeStd bool, todo func()) int {
	var ret uintptr
	var err syscall.Errno
	ret, _, err = syscall.Syscall(syscall.SYS_FORK, 0, 0, 0)
	if err != 0 {
		return -1
	}
	switch ret {
	case 0:
		break
	default:
		return 0
	}
	if pid, err := syscall.Setsid(); err != nil || pid == -1 {
		fmt.Println("failed in setsid")
		return -1
	}
	if chdir {
		os.Chdir("/")
	}
	if closeStd {
		syscall.Close(int(os.Stdin.Fd()))
		syscall.Close(int(os.Stdout.Fd()))
		syscall.Close(int(os.Stderr.Fd()))
	}

	todo()

	return 0
}
