package fcexamplehandler

import (
	"testing"

	"git.inverse.ca/inverse/fingerbank-collector/common"
)

func TestExtractUserAgent(t *testing.T) {
	h := NewExampleHandler(fccommon.NewFingerbankCollector("")).(*ExampleHandler)

	var result string

	// valid user agent after the MAC
	b := []byte("001122334455my-user-agent")
	result = h.extractUserAgent(b)

	if result != "my-user-agent" {
		t.Error("Error parsing user agent from payload")
	}

	// empty user agent after the MAC
	b = []byte("001122334455")
	result = h.extractUserAgent(b)

	if result != "" {
		t.Error("Error parsing user agent from payload")
	}
}

func TestExtractMAC(t *testing.T) {
	h := NewExampleHandler(fccommon.NewFingerbankCollector("")).(*ExampleHandler)

	var result string

	b := []byte("001122334455my-user-agent")
	result = h.extractMAC(b)

	if result != "001122334455" {
		t.Error("Error parsing MAC address from payload")
	}
}

func TestHandle(t *testing.T) {
	h := NewExampleHandler(fccommon.NewFingerbankCollector("")).(*ExampleHandler)

	b := []byte("001122334455my-user-agent")
	h.Handle(fccommon.PacketInfo{SrcIP: "1.2.3.4"}, b, []byte(""))

	mac, err := h.Collector.IP2MAC("1.2.3.4")

	if err != nil || mac != "001122334455" {
		t.Error("Handle didn't create the proper IP2MAC entry")
	}

	ip, err := h.Collector.MAC2IP("001122334455")

	if err != nil || ip != "1.2.3.4" {
		t.Error("Handle didn't create the proper MAC2IP entry")
	}

	ed, err := h.Collector.FindEndpointDataIP("1.2.3.4")

	if err != nil {
		t.Error("Example handler didn't create an EndpointData entry")
	}

	if ed.MostAccurateUserAgent != "my-user-agent" {
		t.Error("Example handler didn't find the user agent correctly")
	}

	if ed.DestinationHosts["otbm.com"][0] != 1 {
		t.Error("Example handler didn't find the destination host correctly")
	}
}

func BenchmarkHandle(b *testing.B) {
	h := NewExampleHandler(fccommon.NewFingerbankCollector("")).(*ExampleHandler)

	l4Payload := []byte("001122334455my-user-agent")

	for i := 0; i < b.N; i++ {
		h.Handle(fccommon.PacketInfo{SrcIP: "1.2.3.4"}, l4Payload, []byte(""))
	}

}
