package fctlshandler

import (
	"git.inverse.ca/inverse/fingerbank-collector/elru"
	"git.inverse.ca/inverse/fingerbank-collector/timedlock"
	"github.com/inverse-inc/go-utils/sharedutils"
)

type IPDomainTracker struct {
	cache *elru.EncodableLRU
	lock  *timedlock.RWLock
}

func NewIPDomainTracker(maxSize int) *IPDomainTracker {
	cache, err := elru.NewEncodableLRU(maxSize)
	sharedutils.CheckError(err)

	ipdt := &IPDomainTracker{
		cache: cache,
		lock:  timedlock.NewRWLock(),
	}
	ipdt.lock.Panic = true

	return ipdt
}

func (ipdt *IPDomainTracker) Add(clientIpPort string, serverIpPort string, host string, clientSig string) {
	lockid := ipdt.lock.Lock()
	defer ipdt.lock.Unlock(lockid)

	var m map[string]map[string]string
	var found bool
	if m, found = ipdt.Get(clientIpPort); !found {
		m = map[string]map[string]string{}
	}

	if m[serverIpPort] == nil {
		m[serverIpPort] = map[string]string{}
	}

	m[serverIpPort][host] = clientSig

	ipdt.cache.Add(clientIpPort, m)
}

func (ipdt *IPDomainTracker) Get(host string) (map[string]map[string]string, bool) {
	if o, found := ipdt.cache.Get(host); found {
		return o.(map[string]map[string]string), found
	} else {
		return map[string]map[string]string{}, found
	}
}

func (ipdt *IPDomainTracker) RLock() uint64 {
	return ipdt.lock.RLock()
}

func (ipdt *IPDomainTracker) RUnlock(lockid uint64) {
	ipdt.lock.RUnlock(lockid)
}
