//go:build classic_pf

package packetfence

import (
	"context"
	"fmt"
	"os"
	"os/exec"
	"regexp"

	"github.com/inverse-inc/packetfence/go/pfconfigdriver"
	ps "github.com/mitchellh/go-ps"
)

func RunsOnPacketFence(ctx context.Context) bool {
	return ConnectPfconfig(ctx) &&
		Pfdir(ctx) &&
		ValidatePackages(ctx) &&
		Pfcmd(ctx) &&
		Processes(ctx)
}

func ConnectPfconfig(ctx context.Context) bool {
	defer func() {
		if r := recover(); r != nil {
			fmt.Println("Failed check cb8f0450-e1b7-11e7-80c1-9a214cf093ae:w")
		}
	}()

	general := pfconfigdriver.PfConfGeneral{}
	pfconfigdriver.FetchDecodeSocket(ctx, &general)

	if general.Domain == "" {
		fmt.Println("Failed check c4abd654-e1b7-11e7-80c1-9a214cf093ae")
		return false
	} else {
		return true
	}
}

func Pfdir(ctx context.Context) bool {
	if _, err := os.Stat("/usr/local/pf"); os.IsNotExist(err) {
		fmt.Println("Failed check baad3da0-e1b7-11e7-80c1-9a214cf093ae")
		return false
	}
	return true
}

func Pfcmd(ctx context.Context) bool {
	version, _ := exec.Command("/usr/local/pf/bin/pfcmd", "version").Output()
	match, _ := regexp.Match(`^PacketFence [0-9.]+`, version)
	return match
}

func Processes(ctx context.Context) bool {
	checks := map[*regexp.Regexp]bool{
		regexp.MustCompile(`pfmon|pfcron`): false,
		regexp.MustCompile(`pfqueue`):      false,
		regexp.MustCompile(`pfconfig`):     false,
		regexp.MustCompile(`radius`):       false,
	}

	processes, _ := ps.Processes()

	for _, process := range processes {
		for re, alreadyMatched := range checks {
			if !alreadyMatched && re.MatchString(process.Executable()) {
				checks[re] = true
			}
		}
	}

	for _, matched := range checks {
		if !matched {
			fmt.Println("Failed check c9f213e6-ddfc-4bdc-9fe1-e26e04a21ecd")
			return false
		}
	}
	return true
}

func ValidatePackages(ctx context.Context) bool {
	return checkPackageInstall(ctx, "packetfence") &&
		checkPackageInstall(ctx, "fingerbank-collector")
}
