package main

import (
	"flag"
	"net"

	"github.com/inverse-inc/go-utils/sflow"
	"github.com/inverse-inc/go-utils/sflow/processor"
	"github.com/inverse-inc/go-utils/sharedutils"
)

type samplesHandler struct {
}

func (h *samplesHandler) HandleSamples(header *sflow.Header, samples []sflow.Sample) {
	for _, s := range samples {
		flowRecords := []sflow.Flow{}
		switch v := s.(type) {
		default:
			continue
		case *sflow.FlowSample:
			flowRecords = v.Records
		case *sflow.FlowSampleExpanded:
			flowRecords = v.Records
		}

		if flowRecords != nil {
			for _, flow := range flowRecords {
				var ip4 *sflow.SampledIPV4
				switch v := flow.(type) {
				case *sflow.SampledIPV4:
					ip4 = v
				case *sflow.SampledHeader:
					ip4 = v.SampledIPv4()
					if ip4 != nil && (ip4.Protocol != 6 && ip4.Protocol != 17) {
						ip4 = nil
					}
				}
				if ip4 != nil {
					var proto string
					if ip4.Protocol == 6 {
						proto = "tcp"
					} else if ip4.Protocol == 17 {
						proto = "udp"
					} else {
						proto = "*"
					}
					//fmt.Println("Handling IP packet from sflow", net.IP(ip4.SrcIP[:]), net.IP(ip4.DstIP[:]), int(ip4.SrcPort), int(ip4.DstPort))
					collector.HandleIPPacket(proto, net.IP(ip4.SrcIP[:]), net.IP(ip4.DstIP[:]), int(ip4.SrcPort), int(ip4.DstPort))
				}
			}
		}
	}
}

var sflowHandler = samplesHandler{}

type SFlowService struct {
	processor processor.Processor
	disable   bool
}

var sflowService = SFlowService{
	processor: processor.Processor{
		Handler: &sflowHandler,
	},
	disable: false,
}

func (s *SFlowService) Start() {
	if !sflowService.disable {
		conn, err := net.ListenUDP("udp", &net.UDPAddr{
			Port: 6343,
			IP:   net.ParseIP("0.0.0.0"),
		})
		sharedutils.CheckError(err)
		sflowService.processor.Conn = conn
		s.processor.Start()
	}
}

func (s *SFlowService) Stop() {
	if !sflowService.disable {
		s.processor.StopAndWait()
	}
}

func init() {
	flag.BoolVar(
		&sflowService.disable,
		"disable-sflow-service",
		(sharedutils.IsEnabled(sharedutils.EnvOrDefault("COLLECTOR_DISABLE_SFLOW_SERVICE", "false")) || sflowService.disable),
		"Disables the sflow service (environment variable: COLLECTOR_DISABLE_SFLOW_SERVICE)",
	)

}
